@@ -62,18 +62,20 @@ enum
 {
 	PROP_0,
 	PROP_DICT,
-	PROP_BLOCK_CHANGED
+	PROP_BLOCK_CHANGED,
+	PROP_ID_STRING
 };
 
 
 struct _GnomeDbBasePrivate
 {
-	GnomeDbDict            *dict;     /* property: NOT NULL to use GnomeDbBase object */
+	GnomeDbDict       *dict;     /* property: NOT NULL to use GnomeDbBase object */
 	guint              id;	     /* 0 or UNIQUE, comes from DictManager->id_serial */
 
 	gchar             *name;
 	gchar             *descr;
 	gchar             *owner;
+	gchar             *id_string;
 	
 	gboolean           nullified;/* TRUE if the object has been "nullified" */
 	gboolean           changed_locked;
@@ -178,10 +180,14 @@ gnome_db_base_class_init (GnomeDbBaseClass * class)
 	object_class->set_property = gnome_db_base_set_property;
 	object_class->get_property = gnome_db_base_get_property;
 	g_object_class_install_property (object_class, PROP_DICT,
-					 g_param_spec_pointer ("dict", NULL, NULL, (G_PARAM_READABLE | G_PARAM_WRITABLE)));
+					 g_param_spec_pointer ("dict", "GnomeDbDict", NULL, 
+							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_BLOCK_CHANGED,
-                                         g_param_spec_boolean ("changed_blocked", NULL, NULL, FALSE,
+                                         g_param_spec_boolean ("changed_blocked", "Block changed signal", NULL, FALSE,
                                                                G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (object_class, PROP_ID_STRING,
+                                         g_param_spec_string ("string_id", "Identifier string", NULL, NULL,
+							      G_PARAM_READABLE | G_PARAM_WRITABLE));
 }
 
 static void
@@ -195,6 +201,7 @@ gnome_db_base_init (GnomeDbBase * base)
 	base->priv->name = NULL;
 	base->priv->descr = NULL;
 	base->priv->owner = NULL;
+	base->priv->id_string = NULL;
 
 	base->priv->changed_locked = FALSE;
 }
@@ -264,6 +271,8 @@ gnome_db_base_finalize (GObject   * object)
 			g_free (base->priv->descr);
 		if (base->priv->owner)
 			g_free (base->priv->owner);
+		if (base->priv->id_string)
+			g_free (base->priv->id_string);
 
 		g_free (base->priv);
 		base->priv = NULL;
@@ -276,9 +285,9 @@ gnome_db_base_finalize (GObject   * object)
 
 static void 
 gnome_db_base_set_property (GObject              *object,
-		      guint                 param_id,
-		      const GValue         *value,
-		      GParamSpec           *pspec)
+			    guint                 param_id,
+			    const GValue         *value,
+			    GParamSpec           *pspec)
 {
 	gpointer ptr;
 	GnomeDbBase *base;
@@ -295,15 +304,22 @@ gnome_db_base_set_property (GObject              *object,
 				gnome_db_base_block_changed (base);
 			else
 				gnome_db_base_unblock_changed (base);
+			break;
+		case PROP_ID_STRING:
+			if (base->priv->id_string)
+				g_free (base->priv->id_string);
+			if (g_value_get_string (value))
+				base->priv->id_string = g_strdup (g_value_get_string (value));
+			break;
 		}
 	}
 }
 
 static void
 gnome_db_base_get_property (GObject              *object,
-		      guint                 param_id,
-		      GValue               *value,
-		      GParamSpec           *pspec)
+			    guint                 param_id,
+			    GValue               *value,
+			    GParamSpec           *pspec)
 {
 	GnomeDbBase *base;
 
@@ -315,6 +331,10 @@ gnome_db_base_get_property (GObject              *object,
 			break;
 		case PROP_BLOCK_CHANGED:
 			g_value_set_boolean (value, base->priv->changed_locked);
+			break;
+		case PROP_ID_STRING:
+			g_value_set_string (value, base->priv->id_string);
+			break;
 		}
 	}
 }
