@@ -1,6 +1,6 @@
 /* gnome-db-database.c
  *
- * Copyright (C) 2003 - 2004 Vivien Malerba
+ * Copyright (C) 2003 - 2005 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -1419,9 +1419,11 @@ gnome_db_database_get_field_by_name (GnomeDbDatabase *mgdb, const gchar *fullnam
 	if (ref_table) {
 		gpointer data;
 		ptr = strtok_r (NULL, ".", &tok);
-		data = gnome_db_entity_get_field_by_name (GNOME_DB_ENTITY (ref_table), ptr);
-		if (data)
-			ref_field = GNOME_DB_FIELD (data);
+		if (ptr) {
+			data = gnome_db_entity_get_field_by_name (GNOME_DB_ENTITY (ref_table), ptr);
+			if (data)
+				ref_field = GNOME_DB_FIELD (data);
+		}
 	}
 	g_free (str);
 
