@@ -1613,7 +1613,7 @@ gnome_db_dict_get_database (GnomeDbDict *dict)
  * gnome_db_dict_dump
  * @dict: a #GnomeDbDict object
  *
- * Dumps the whole dictionnary managed by the GnomeDbDict object
+ * Dumps the whole dictionary managed by the GnomeDbDict object
  */
 void
 gnome_db_dict_dump (GnomeDbDict *dict)
@@ -1674,12 +1674,12 @@ gnome_db_dict_dump (GnomeDbDict *dict)
  * @app_id: an extra identification, or %NULL
  * @error: location to store error, or %NULL
  *
- * Get the prefered filename which represents the data dictionnary associated to the @datasource data source.
+ * Get the prefered filename which represents the data dictionary associated to the @datasource data source.
  * Using the returned value in conjunction with gnome_db_dict_load_xml_file() and gnome_db_dict_save_xml_file() has
  * the advantage of letting the library handle file naming onventions.
  *
  * The @app_id argument allows to give an extra identification to the request, when some special features
- * must be saved but not interfere with the default dictionnary.
+ * must be saved but not interfere with the default dictionary.
  *
  * Returns: a new string
  */
@@ -1693,17 +1693,17 @@ gnome_db_dict_compute_xml_filename (GnomeDbDict *dict, const gchar *datasource,
  *      an argument */
 
 /* same as for libgda */
-#define LIBGDA_USER_DICTIG_DIR G_DIR_SEPARATOR_S ".libgda"
+#define LIBGDA_USER_CONFIG_DIR G_DIR_SEPARATOR_S ".libgda"
 
 	g_return_val_if_fail (dict && IS_GNOME_DB_DICT (dict), NULL);
 	g_return_val_if_fail (dict->priv, NULL);
 	g_return_val_if_fail (datasource && *datasource, NULL);
 
 	if (!app_id)
-		str = g_strdup_printf ("%s%sDICT_%s_default.xml", g_get_home_dir (), LIBGDA_USER_DICTIG_DIR G_DIR_SEPARATOR_S,
+		str = g_strdup_printf ("%s%sDICT_%s_default.xml", g_get_home_dir (), LIBGDA_USER_CONFIG_DIR G_DIR_SEPARATOR_S,
 				       datasource);
 	else
-		str = g_strdup_printf ("%s%sDICT_%s_%s.xml", g_get_home_dir (), LIBGDA_USER_DICTIG_DIR G_DIR_SEPARATOR_S,
+		str = g_strdup_printf ("%s%sDICT_%s_%s.xml", g_get_home_dir (), LIBGDA_USER_CONFIG_DIR G_DIR_SEPARATOR_S,
 				       datasource, app_id);
 
 	/* create an empty file with that name */
@@ -1711,7 +1711,7 @@ gnome_db_dict_compute_xml_filename (GnomeDbDict *dict, const gchar *datasource,
 		gchar *dirpath;
 		FILE *fp;
 		
-		dirpath = g_strdup_printf ("%s%s", g_get_home_dir (), LIBGDA_USER_DICTIG_DIR);
+		dirpath = g_strdup_printf ("%s%s", g_get_home_dir (), LIBGDA_USER_CONFIG_DIR);
 		if (!g_file_test (dirpath, G_FILE_TEST_IS_DIR)){
 			if (mkdir (dirpath, 0700)) {
 				g_set_error (error,
@@ -1728,7 +1728,7 @@ gnome_db_dict_compute_xml_filename (GnomeDbDict *dict, const gchar *datasource,
 /* 			g_set_error (error, */
 /* 				     GNOME_DB_DICT_ERROR, */
 /* 				     GNOME_DB_DICT_FILE_LOAD_ERROR, */
-/* 				     _("Unable to create the dictionnary file %s"), str); */
+/* 				     _("Unable to create the dictionary file %s"), str); */
 /* 			with_error = TRUE; */
 /* 		} */
 /* 		else */
