@@ -120,13 +120,10 @@ struct _GnomeDbGridPriv
 	GSList            *columns_data; /* list of ColumnData */
 	gboolean           internal_params_change;
 
-	gint               sample_first_row;
-	gint               sample_last_row;
-	gint               sample_size;
-
 	GtkWidget         *title;
 	GtkWidget         *scroll;
 	GtkWidget         *treeview;
+	gboolean           titles_shown;
 
 	gboolean           default_show_info_cell;
 	gboolean           default_column_editable;
@@ -137,6 +134,8 @@ struct _GnomeDbGridPriv
 	GtkWidget         *widget_actions;
 	GtkWidget         *widget_current_sample;
 	GtkWidget         *widget_group;
+
+	gint               export_type; /* used by the export dialog */
 };
 
 /* get a pointer to the parents to be able to call their destructor */
@@ -276,15 +275,14 @@ gnome_db_grid_init (GnomeDbGrid * wid)
 	wid->priv->columns_data = NULL;
 	wid->priv->internal_params_change = FALSE;
 
-	wid->priv->sample_first_row = 0;
-	wid->priv->sample_last_row = 0;
-	wid->priv->sample_size = 100;
-
 	wid->priv->treeview = NULL;
+	wid->priv->titles_shown = TRUE;
 
 	wid->priv->mode = 0;
 	wid->priv->tooltips = NULL;
 
+	wid->priv->export_type = 2;
+
 	/* title */
 	wid->priv->title = gnome_db_gray_bar_new (_("No title"));
         gtk_box_pack_start (GTK_BOX (wid), wid->priv->title, FALSE, TRUE, 2);
@@ -610,7 +608,19 @@ gnome_db_grid_get_selection (GnomeDbGrid *grid)
 void
 gnome_db_grid_foreach_selected (GnomeDbGrid *grid, GnomeDbGridForeachFunc foreach_func, gpointer user_data)
 {
-	TO_IMPLEMENT;
+	GList *selected_rows, *list;
+
+	g_return_if_fail (grid && IS_GNOME_DB_GRID (grid));
+	g_return_if_fail (grid->priv);
+	g_return_if_fail (foreach_func);
+
+	selected_rows = gnome_db_grid_get_selection (grid);
+	list = selected_rows;
+	while (list) {
+		foreach_func (grid, (gint) (list->data), user_data);
+		list = g_list_next (list);
+	}
+	g_list_free (selected_rows);
 }
 
 
@@ -620,7 +630,10 @@ gnome_db_grid_foreach_selected (GnomeDbGrid *grid, GnomeDbGridForeachFunc foreac
 void
 gnome_db_grid_select_all (GnomeDbGrid *grid)
 {
-	TO_IMPLEMENT;
+	GtkTreeSelection *selection;
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid->priv->treeview));
+	
+	gtk_tree_selection_select_all (selection);
 }
 
 /**
@@ -629,7 +642,10 @@ gnome_db_grid_select_all (GnomeDbGrid *grid)
 void
 gnome_db_grid_unselect_all (GnomeDbGrid *grid)
 {
-	TO_IMPLEMENT;
+	GtkTreeSelection *selection;
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid->priv->treeview));
+	
+	gtk_tree_selection_unselect_all (selection);
 }
 
 /* 
@@ -643,15 +659,18 @@ work_param_changed_cb (GnomeDbParameter *param, GnomeDbGrid *grid)
 	GtkTreeSelection *selection;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
+	GList *sel_rows, *cur_row;
 
 	if (!grid->priv->treeview)
 		return;
+	if (grid->priv->internal_params_change)
+		return;
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid->priv->treeview));
-	if (!grid->priv->internal_params_change && gtk_tree_selection_get_selected (selection, &model, &iter)) {
-		GtkTreePath *path;
-
-		path = gtk_tree_model_get_path (model, &iter);
+	sel_rows = gtk_tree_selection_get_selected_rows (selection, &model);
+	cur_row = sel_rows;
+	while (cur_row) {
+		gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) (cur_row->data));
 		list = grid->priv->data_set->parameters;
 		while (list) {
 			GSList *depend = gnome_db_parameter_get_dependencies (GNOME_DB_PARAMETER (list->data));
@@ -665,8 +684,12 @@ work_param_changed_cb (GnomeDbParameter *param, GnomeDbGrid *grid)
 			}
 			list = g_slist_next (list);
 		}
-		gtk_tree_path_free (path);
+
+		cur_row = g_list_next (cur_row);
 	}
+	g_list_foreach (sel_rows, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (sel_rows);
+	modif_buttons_update (grid);
 }
 
 static void
@@ -790,7 +813,10 @@ static void     cell_renderer_value_set_attributes (GtkTreeViewColumn *tree_colu
 						    GtkTreeModel *tree_model, GtkTreeIter *iter, GnomeDbGrid *grid);
 static void     cell_renderer_info_set_attributes  (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
 						    GtkTreeModel *tree_model, GtkTreeIter *iter, GnomeDbGrid *grid);
+
 static gboolean tree_view_event_cb (GtkWidget *treeview, GdkEvent *event, GnomeDbGrid *grid);
+static gint     tree_view_popup_button_pressed_cb (GtkWidget *widget, GdkEventButton *event, GnomeDbGrid *grid);
+
 static void     tree_view_selection_changed_cb (GtkTreeSelection *selection, GnomeDbGrid *grid);
 static void     tree_view_row_activated_cb     (GtkTreeView *tree_view, GtkTreePath *path, 
 						GtkTreeViewColumn *column, GnomeDbGrid *grid);
@@ -826,11 +852,15 @@ create_new_tree_view (GnomeDbGrid *grid)
 	grid->priv->treeview = GTK_WIDGET (tree_view);
 	gtk_container_add (GTK_CONTAINER (grid->priv->scroll), grid->priv->treeview);
 	gtk_widget_show (grid->priv->treeview);
+
 	g_signal_connect (G_OBJECT (tree_view), "event",
 			  G_CALLBACK (tree_view_event_cb), grid);
+	g_signal_connect (G_OBJECT (tree_view), "button_press_event",
+                          G_CALLBACK (tree_view_popup_button_pressed_cb), grid);
 
 	/* selection and signal handling */
 	selection = gtk_tree_view_get_selection (tree_view);
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 	g_signal_connect (G_OBJECT (selection), "changed",
 			  G_CALLBACK (tree_view_selection_changed_cb), grid);
 	g_signal_connect (G_OBJECT (tree_view), "row_activated",
@@ -1297,25 +1327,18 @@ data_cell_values_changed (GtkCellRenderer *renderer, const gchar *path,
 static gboolean
 set_iter_from_path (GnomeDbGrid *grid, const gchar *path, GtkTreeIter *iter)
 {
-	if (path) {
-		GtkTreePath *treepath;
+	GtkTreePath *treepath;
 
-		treepath = gtk_tree_path_new_from_string (path);
-		if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (grid->priv->proxy), iter, treepath)) {
-			gtk_tree_path_free (treepath);
-			g_warning ("Can't get iter for path %s", path);
-			return FALSE;
-		}
+	g_assert (path);
+
+	treepath = gtk_tree_path_new_from_string (path);
+	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (grid->priv->proxy), iter, treepath)) {
 		gtk_tree_path_free (treepath);
+		g_warning ("Can't get iter for path %s", path);
+		return FALSE;
 	}
-	else {
-		GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid->priv->treeview));
-		if (!gtk_tree_selection_get_selected (selection, NULL, iter)) {
-			g_warning ("Can't get iter for current selection");
-			return FALSE;
-		}
-	}
-
+	gtk_tree_path_free (treepath);
+	
 	return TRUE;
 }
 
@@ -1527,16 +1550,25 @@ tree_view_event_cb (GtkWidget *treeview, GdkEvent *event, GnomeDbGrid *grid)
 		/* DELETE to delete the selected row */
 		if (ekey->keyval == GDK_Delete) {
 			GtkTreeIter iter;
-			GtkTreeSelection *select;
+			GtkTreeSelection *selection;
+			GtkTreeModel *model;
+			GList *sel_rows, *cur_row;
 
-			select = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid->priv->treeview));
-			if (gtk_tree_selection_get_selected (select, NULL, &iter)) {
+			selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid->priv->treeview));
+			sel_rows = gtk_tree_selection_get_selected_rows (selection, &model);
+			cur_row = sel_rows;
+			while (cur_row) {
+				gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) (cur_row->data));
 				if (((ekey->state & modifiers) == GDK_SHIFT_MASK) || 
 				    ((ekey->state & modifiers) == GDK_CONTROL_MASK))
 					gnome_db_data_proxy_undelete (grid->priv->proxy, &iter);
 				else
 					gnome_db_data_proxy_delete (grid->priv->proxy, &iter);
+				cur_row = g_list_next (cur_row);
 			}
+			g_list_foreach (sel_rows, (GFunc) gtk_tree_path_free, NULL);
+			g_list_free (sel_rows);
+			
 			done = TRUE;
 		}
 	}
@@ -1544,6 +1576,298 @@ tree_view_event_cb (GtkWidget *treeview, GdkEvent *event, GnomeDbGrid *grid)
 	return done;
 }
 
+static void menu_select_all_cb (GtkWidget *widget, GnomeDbGrid *grid);
+static void menu_unselect_all_cb (GtkWidget *widget, GnomeDbGrid *grid);
+static void menu_show_columns_cb (GtkWidget *widget, GnomeDbGrid *grid);
+static void menu_save_as_cb (GtkWidget *widget, GnomeDbGrid *grid);
+
+static gint
+tree_view_popup_button_pressed_cb (GtkWidget *widget, GdkEventButton *event, GnomeDbGrid *grid)
+{
+	GtkWidget *menu;
+	GtkTreeSelection *selection;
+	GtkSelectionMode sel_mode;
+
+        if (event->button != 3)
+                return FALSE;
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid->priv->treeview));
+	sel_mode = gtk_tree_selection_get_mode (selection);
+
+        /* create the menu */
+        menu = gtk_menu_new ();
+	if (sel_mode == GTK_SELECTION_MULTIPLE)
+		gtk_menu_append (GTK_MENU (menu),
+				 gnome_db_new_menu_item (_("Select _All"), FALSE, 
+							 G_CALLBACK (menu_select_all_cb), grid));
+
+	if ((sel_mode == GTK_SELECTION_SINGLE) || (sel_mode == GTK_SELECTION_MULTIPLE))
+		gtk_menu_append (GTK_MENU (menu),
+				 gnome_db_new_menu_item (_("_Clear Selection"), FALSE,
+							 G_CALLBACK (menu_unselect_all_cb), grid));
+        gtk_menu_append (GTK_MENU (menu),
+			 gnome_db_new_check_menu_item (_("Show Column _Titles"),
+						       grid->priv->titles_shown,
+						       G_CALLBACK (menu_show_columns_cb), grid));
+
+	if (sel_mode != GTK_SELECTION_NONE) {
+		gtk_menu_append (GTK_MENU (menu), gtk_separator_menu_item_new ());
+		gtk_menu_append (GTK_MENU (menu), gnome_db_new_menu_item (GTK_STOCK_SAVE_AS, TRUE,
+					       G_CALLBACK (menu_save_as_cb), grid));
+	}
+
+	/* allow listeners to add their custom menu items */
+        g_signal_emit (G_OBJECT (grid), gnome_db_grid_signals [CREATE_POPUP_MENU], 0, GTK_MENU (menu));
+        gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, event->button, event->time);
+        gtk_widget_show_all (menu);
+	
+        return TRUE;
+}
+
+static void
+menu_select_all_cb (GtkWidget *widget, GnomeDbGrid *grid)
+{
+        gnome_db_grid_select_all (grid);
+}
+
+static void
+menu_unselect_all_cb (GtkWidget *widget, GnomeDbGrid *grid)
+{
+        gnome_db_grid_unselect_all (grid);
+}
+
+static void
+menu_show_columns_cb (GtkWidget *widget, GnomeDbGrid *grid)
+{
+        GtkCheckMenuItem *item;
+
+        item = (GtkCheckMenuItem *) widget;
+
+        g_return_if_fail (GTK_IS_CHECK_MENU_ITEM (item));
+	grid->priv->titles_shown = gtk_check_menu_item_get_active (item);
+
+	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (grid->priv->treeview), grid->priv->titles_shown);
+}
+
+static void save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbGrid *grid);
+
+static void
+menu_save_as_cb (GtkWidget *widget, GnomeDbGrid *grid)
+{
+	GtkWidget *dialog;
+	GtkWidget *label;
+	GtkWidget *filename;
+	GtkWidget *types;
+	GtkWidget *hbox;
+	char *str;
+
+	/* create dialog box */
+	dialog = gtk_dialog_new_with_buttons (_("Saving Data"),
+					      GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (grid))), 0,
+					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+					      NULL);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
+
+	str = g_strdup_printf ("<big><b>%s:</b></big>\n%s", _("Saving data to a file"), 
+			       _("The data will be exported without any of the modifications which may "
+				 "have been made and have not been comitted."));
+        label = gtk_label_new ("");
+        gtk_label_set_markup (GTK_LABEL (label), str);
+        gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+        g_free (str);
+        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), label, FALSE, TRUE, 2);
+
+	str = g_strdup_printf ("<b>%s:</b>", _("File name"));
+        label = gtk_label_new ("");
+        gtk_label_set_markup (GTK_LABEL (label), str);
+        gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
+        g_free (str);
+        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), label, FALSE, TRUE, 2);
+
+	hbox = gtk_hbox_new (FALSE, 0); /* HIG */
+        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, FALSE, FALSE, 5);
+        gtk_widget_show (hbox);
+        label = gtk_label_new ("    ");
+        gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+        gtk_widget_show (label);
+
+	filename = gtk_file_chooser_widget_new (GTK_FILE_CHOOSER_ACTION_SAVE);
+	g_object_set_data (G_OBJECT (dialog), "filename", filename);
+	gtk_box_pack_start (GTK_BOX (hbox), filename, TRUE, TRUE, 0);
+
+	str = g_strdup_printf ("<b>%s:</b>", _("File type"));
+        label = gtk_label_new ("");
+        gtk_label_set_markup (GTK_LABEL (label), str);
+        gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
+        g_free (str);
+        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), label, FALSE, TRUE, 2);
+
+	hbox = gtk_hbox_new (FALSE, 0); /* HIG */
+        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, FALSE, FALSE, 5);
+        gtk_widget_show (hbox);
+        label = gtk_label_new ("    ");
+        gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+        gtk_widget_show (label);
+
+	types = gtk_combo_box_new_text ();
+	gtk_box_pack_start (GTK_BOX (hbox), types, TRUE, TRUE, 0);
+	g_object_set_data (G_OBJECT (dialog), "types", types);
+
+	gtk_combo_box_append_text (GTK_COMBO_BOX (types), _("Tab-delimited"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX (types), _("Comma-delimited"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX (types), _("XML"));
+	gtk_combo_box_set_active (GTK_COMBO_BOX (types), grid->priv->export_type);
+	
+	/* run the dialog */
+	g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (save_as_response_cb), grid);
+	gtk_widget_show_all (dialog);
+}
+
+static gboolean confirm_file_overwrite (GtkWindow *parent, const gchar *path);
+
+static void
+save_as_response_cb (GtkDialog *dialog, guint response_id, GnomeDbGrid *grid)
+{
+	GtkWidget *types;
+	gint export_type;
+	GtkWidget *filename;
+	
+	if (response_id == GTK_RESPONSE_OK) {
+		gchar *body;
+		gchar *path;
+		GList *columns, *list;
+		gint *cols, nb_cols;
+
+		types = g_object_get_data (G_OBJECT (dialog), "types");
+		filename = g_object_get_data (G_OBJECT (dialog), "filename");
+
+		/* output columns computation */
+		columns = gtk_tree_view_get_columns (GTK_TREE_VIEW (grid->priv->treeview));
+		cols = g_new (gint, gda_data_model_get_n_columns (GDA_DATA_MODEL (grid->priv->data_model)));
+		nb_cols = 0;
+		list = columns;
+		while (list) {
+			if (gtk_tree_view_column_get_visible (GTK_TREE_VIEW_COLUMN (list->data))) {
+				GnomeDbDataSetNode *node;
+
+				node = g_object_get_data (G_OBJECT (list->data), "context-node");
+				if (node->param) {
+					cols [nb_cols] = gnome_db_data_model_get_column_at_param (grid->priv->data_model,
+												  grid->priv->data_set,
+												  node->param);
+					nb_cols ++;
+				}
+				else {
+					GSList *params = node->params;
+					while (params) {
+						cols [nb_cols] = gnome_db_data_model_get_column_at_param (grid->priv->data_model,
+										              grid->priv->data_set,
+											      GNOME_DB_PARAMETER (params->data));
+						nb_cols ++;
+						params = g_slist_next (params);
+					}
+				}
+			}
+			list = g_list_next (list);
+		}
+		g_list_free (columns);
+		
+		/* Actual ouput computations */
+		export_type = gtk_combo_box_get_active (GTK_COMBO_BOX (types));
+		grid->priv->export_type = export_type;
+		switch (export_type) {
+		case 0:
+			body = gda_data_model_to_text_separated (GDA_DATA_MODEL (grid->priv->data_model), 
+								 cols, nb_cols, '\t');
+			break;
+		case 1:
+			body = gda_data_model_to_text_separated (GDA_DATA_MODEL (grid->priv->data_model), 
+								 cols, nb_cols, ',');
+			break;
+		case 2:
+			body = gda_data_model_to_xml (GDA_DATA_MODEL (grid->priv->data_model), cols, nb_cols,
+						      gnome_db_base_get_name (GNOME_DB_BASE (grid->priv->data_model)));
+			break;
+		default:
+			g_assert_not_reached ();
+			break;
+		}
+		g_free (cols);
+
+		/* saving */
+		if (body) {
+			path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (filename));
+			if (path) {
+				if (g_file_test (path, G_FILE_TEST_EXISTS)) {
+					if (! confirm_file_overwrite (GTK_WINDOW (dialog), path)) {
+						g_free (body);
+						g_free (path);
+						return;
+					}
+				}
+
+				if (! gda_file_save (path, body, strlen (body))) {
+					gnome_db_show_error (_("Could not save file %s"), path);
+					g_free (body);
+					g_free (path);
+					return;
+				}
+				g_free (path);
+			} 
+			else {
+				gnome_db_show_error (_("You must specify a file name"));
+				g_free (body);
+				return;
+			}
+			g_free (body);
+		} else
+			gnome_db_show_error (_("Got empty file while converting the data"));
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
+static gboolean
+confirm_file_overwrite (GtkWindow *parent, const gchar *path)
+{
+	GtkWidget *dialog, *button;
+	gboolean yes;
+	gchar *msg;
+
+	msg = g_strdup_printf (_("File '%s' already exists.\n"
+				 "Do you want to overwrite it?"), path);
+
+	/* build the dialog */
+	dialog = gnome_db_new_alert (
+		NULL,
+		GTK_MESSAGE_QUESTION,
+		msg,
+		_("If you choose yes, the contents will be lost."));
+	button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
+				      button,
+				      GTK_RESPONSE_NO);
+	gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
+				      gtk_button_new_from_stock (GTK_STOCK_YES),
+				      GTK_RESPONSE_YES);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+					 GTK_RESPONSE_NO);
+
+	/* run the dialog */
+	gtk_widget_show_all (dialog);
+	yes = gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES;
+
+	/* free memory */
+	gtk_widget_destroy (dialog);
+	g_free (msg);
+	return yes;
+}
+
+
 static void
 tree_view_row_activated_cb (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GnomeDbGrid *grid)
 {
@@ -1568,8 +1892,23 @@ tree_view_selection_changed_cb (GtkTreeSelection *selection, GnomeDbGrid *grid)
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	gboolean row_selected = FALSE;
+	gint has_selection;
 	
-	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+	if (gtk_tree_selection_get_mode (selection) == GTK_SELECTION_MULTIPLE) {
+		has_selection = gtk_tree_selection_count_selected_rows (selection);
+		if (has_selection == 1) {
+			GList *sel_rows;
+
+			sel_rows = gtk_tree_selection_get_selected_rows (selection, &model);
+			has_selection = gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) (sel_rows->data)) ? 1 : 0;
+			g_list_foreach (sel_rows, (GFunc) gtk_tree_path_free, NULL);
+			g_list_free (sel_rows);
+		}
+	}
+	else
+		has_selection = gtk_tree_selection_get_selected (selection, &model, &iter) ? 1 : 0;
+
+	if (has_selection == 1) {
 		grid->priv->internal_params_change = TRUE;
 		gnome_db_grid_set_params_from_iter (grid, &iter);
 		grid->priv->internal_params_change = FALSE;
@@ -1581,7 +1920,7 @@ tree_view_selection_changed_cb (GtkTreeSelection *selection, GnomeDbGrid *grid)
 		GSList *list = grid->priv->data_set->parameters;
 		while (list) {
 			GnomeDbParameter *tmp;
-			g_object_get (G_OBJECT (list->data), "full_bind", &tmp, NULL);
+			g_object_get (G_OBJECT (list->data), "simple_bind", &tmp, NULL);
 			if (! tmp) 
 				gnome_db_parameter_declare_invalid (GNOME_DB_PARAMETER (list->data));
 			
@@ -1634,9 +1973,10 @@ gnome_db_grid_set_params_from_iter (GnomeDbGrid *grid, GtkTreeIter *iter)
 		guint attributes;
 		GnomeDbParameter *tmp;
 		
-		g_object_get (G_OBJECT (param), "full_bind", &tmp, NULL);
+		g_object_get (G_OBJECT (param), "simple_bind", &tmp, NULL);
 		if (! tmp) {
 			col = gnome_db_data_model_get_column_at_param (grid->priv->data_model, grid->priv->data_set, param);
+			g_assert (col >= 0);
 			gtk_tree_model_get (GTK_TREE_MODEL (grid->priv->proxy), iter, 
 					    col, &value, 
 					    offset + col, &attributes, -1);
@@ -1676,12 +2016,7 @@ gnome_db_grid_set_sample_size (GnomeDbGrid *grid, gint sample_size)
 	g_return_if_fail (grid && IS_GNOME_DB_GRID (grid));
 	g_return_if_fail (grid->priv);
 
-	if (sample_size <0)
-		grid->priv->sample_size = 0;
-	else
-		grid->priv->sample_size = sample_size;
-
-	TO_IMPLEMENT;
+	gnome_db_data_proxy_set_sample_size (grid->priv->proxy, sample_size);
 }
 
 /**
@@ -1696,12 +2031,7 @@ gnome_db_grid_set_sample_start (GnomeDbGrid *grid, gint sample_start)
 	g_return_if_fail (grid && IS_GNOME_DB_GRID (grid));
 	g_return_if_fail (grid->priv);
 	
-	if (sample_start <0)
-		grid->priv->sample_first_row = 0;
-	else
-		grid->priv->sample_first_row = sample_start;
-
-	TO_IMPLEMENT;
+	gnome_db_data_proxy_set_sample_start (grid->priv->proxy, sample_start);
 }
 
 
@@ -1775,6 +2105,7 @@ modif_buttons_make (GnomeDbGrid *grid)
 	gtk_ui_manager_add_ui_from_string (ui, ui_actions_info, -1, NULL);
 	grid->priv->uimanager = ui;
 	grid->priv->widget_actions = gtk_ui_manager_get_widget (ui, "/ToolBar");
+	g_object_set (G_OBJECT (grid->priv->widget_actions), "toolbar-style", GTK_TOOLBAR_ICONS, NULL);
 	gtk_table_attach_defaults (GTK_TABLE (table), grid->priv->widget_actions, 0, 1, 0, 1);
 	gtk_widget_show (grid->priv->widget_actions);
 
@@ -1793,11 +2124,19 @@ modif_buttons_update (GnomeDbGrid *grid)
 	GtkTreeSelection *select;
 	GtkTreeIter iter;
 	GtkTreeModel *model;
-	gboolean changed=FALSE;
-	gboolean has_selection;
+	gboolean changed = FALSE;
+	gint has_selection;
+	GtkSelectionMode sel_mode;
 	gchar *str;
 	GtkAction *action;
 	gint row_count = 0;
+	gboolean params_valid;
+	GnomeDbDataSet *params;
+
+	gint sample_size, sample_first_row, sample_last_row;
+
+	params = gnome_db_data_model_get_params (grid->priv->data_model);
+	params_valid = params ? gnome_db_data_set_is_valid (params) : TRUE;
 
 	if (!grid->priv->treeview || !grid->priv->proxy) {
 		action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionDelete");
@@ -1813,9 +2152,6 @@ modif_buttons_update (GnomeDbGrid *grid)
 		action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionDelete");
 		g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
 		
-		grid->priv->sample_first_row = 0;
-		grid->priv->sample_last_row = 0;
-
 		action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionFirstChunck");
 		g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
 		action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionPrevChunck");
@@ -1833,7 +2169,20 @@ modif_buttons_update (GnomeDbGrid *grid)
 
 	changed = gnome_db_data_proxy_has_been_modified (grid->priv->proxy);
 	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid->priv->treeview));
-	has_selection = gtk_tree_selection_get_selected (select, &model, &iter);
+	sel_mode = gtk_tree_selection_get_mode (select);
+	if (sel_mode == GTK_SELECTION_MULTIPLE) {
+		has_selection = gtk_tree_selection_count_selected_rows (select);
+		if (has_selection == 1) {
+			GList *sel_rows;
+
+			sel_rows = gtk_tree_selection_get_selected_rows (select, &model);
+			has_selection = gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) (sel_rows->data)) ? 1 : 0;
+			g_list_foreach (sel_rows, (GFunc) gtk_tree_path_free, NULL);
+			g_list_free (sel_rows);
+		}
+	}
+	else
+		has_selection = gtk_tree_selection_get_selected (select, &model, &iter) ? 1 : 0;
 
 	/* FIXME: if the working mode is to allow the edition of one line at a time, then 
 	 * we should check that the corresponding INSERT query is valid (or UPDATE, if INSERT is valid then
@@ -1842,64 +2191,62 @@ modif_buttons_update (GnomeDbGrid *grid)
 	g_object_set (G_OBJECT (action), "sensitive", changed ? TRUE : FALSE, NULL);
 
 	action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionReset");
-	g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
+	g_object_set (G_OBJECT (action), "sensitive", params_valid, NULL);
 
 	action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionNew");
 	g_object_set (G_OBJECT (action), "sensitive", 
-		      ! gnome_db_data_proxy_is_read_only (grid->priv->proxy)
-		      /* && gnome_db_data_set_is_valid (grid->priv->data_model))*/, NULL);
-
-	if (has_selection && ! gnome_db_data_proxy_is_read_only (grid->priv->proxy)) {
-		gboolean to_be_deleted = FALSE;
-		gtk_tree_model_get (model, &iter, PROXY_COL_TO_DELETE, &to_be_deleted, -1);
+		      ! gnome_db_data_proxy_is_read_only (grid->priv->proxy) && params_valid, NULL);
 
+	if ((has_selection == 0) || gnome_db_data_proxy_is_read_only (grid->priv->proxy)) {
 		action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionDelete");
-		g_object_set (G_OBJECT (action), "sensitive", !to_be_deleted, NULL);
-		
+		g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
 		action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionUndelete");
-		g_object_set (G_OBJECT (action), "sensitive", to_be_deleted, NULL);
+		g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
 	}
 	else {
-		action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionDelete");
-		g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
-		action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionUndelete");
-		g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
+		if (has_selection > 1) {
+			action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionDelete");
+			g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
+			
+			action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionUndelete");
+			g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
+		}
+		else {
+			gboolean to_be_deleted = FALSE;
+			gtk_tree_model_get (model, &iter, PROXY_COL_TO_DELETE, &to_be_deleted, -1);
+			
+			action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionDelete");
+			g_object_set (G_OBJECT (action), "sensitive", !to_be_deleted, NULL);
+			
+			action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionUndelete");
+			g_object_set (G_OBJECT (action), "sensitive", to_be_deleted, NULL);				
+		}
 	}
 
-	if (grid->priv->sample_size)
-		row_count = gda_data_model_get_n_rows (GDA_DATA_MODEL (grid->priv->data_model));
+	sample_size = gnome_db_data_proxy_get_sample_size (grid->priv->proxy);
+	row_count = gda_data_model_get_n_rows (GDA_DATA_MODEL (grid->priv->data_model));
+	sample_first_row = gnome_db_data_proxy_get_sample_start (grid->priv->proxy);
+	sample_last_row = gnome_db_data_proxy_get_sample_end (grid->priv->proxy);
 
-	action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionFirstChunck");
-	g_object_set (G_OBJECT (action), "sensitive", 
-		      grid->priv->sample_size ? (grid->priv->sample_first_row > 0 ? TRUE : FALSE) : FALSE, 
-		      "visible", grid->priv->mode & GNOME_DB_ACTION_NAVIGATION_ARROWS ? TRUE : FALSE, NULL);
-	action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionPrevChunck");
-	g_object_set (G_OBJECT (action), "sensitive", 
-		      grid->priv->sample_size ? (grid->priv->sample_first_row > 0 ? TRUE : FALSE) : FALSE,  
-		      "visible", grid->priv->mode & GNOME_DB_ACTION_NAVIGATION_ARROWS ? TRUE : FALSE, NULL);
-	action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionNextChunck");
-	g_object_set (G_OBJECT (action), "sensitive", 
-		      grid->priv->sample_size ? (grid->priv->sample_last_row < row_count ? TRUE : FALSE) : FALSE,  
-		      "visible", grid->priv->mode & GNOME_DB_ACTION_NAVIGATION_ARROWS ? TRUE : FALSE, NULL);
-	action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionLastChunck");
-	g_object_set (G_OBJECT (action), "sensitive", 
-		      grid->priv->sample_size ? (grid->priv->sample_last_row < row_count ? TRUE : FALSE) : FALSE,  
-		      "visible", grid->priv->mode & GNOME_DB_ACTION_NAVIGATION_ARROWS ? TRUE : FALSE, NULL);
-	
-	if (grid->priv->sample_last_row == 0)
-		str = g_strdup_printf (_("0 - 0 / 0"));
-	else
-		str = g_strdup_printf ("%d - %d / %d", 
-				       grid->priv->sample_first_row + 1, grid->priv->sample_last_row, row_count);
-	gtk_label_set_text (GTK_LABEL (grid->priv->widget_current_sample), str);
-	g_free (str);
-	gtk_widget_set_sensitive (grid->priv->widget_current_sample, TRUE);
-	if (grid->priv->mode & GNOME_DB_ACTION_NAVIGATION_ARROWS)
-		gtk_widget_show (grid->priv->widget_current_sample);
-	else
-		gtk_widget_hide (grid->priv->widget_current_sample);
-
-	if (1) {
+	if (sample_size > 0) {
+		action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionFirstChunck");
+		g_object_set (G_OBJECT (action), "sensitive", 
+			      sample_first_row > 0 ? TRUE : FALSE, 
+			      "visible", grid->priv->mode & GNOME_DB_ACTION_NAVIGATION_ARROWS ? TRUE : FALSE, NULL);
+		action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionPrevChunck");
+		g_object_set (G_OBJECT (action), "sensitive", 
+			      sample_first_row > 0 ? TRUE : FALSE,  
+			      "visible", grid->priv->mode & GNOME_DB_ACTION_NAVIGATION_ARROWS ? TRUE : FALSE, NULL);
+		action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionNextChunck");
+		g_object_set (G_OBJECT (action), "sensitive", 
+			      sample_last_row < row_count - 1 ? TRUE : FALSE,
+			      "visible", grid->priv->mode & GNOME_DB_ACTION_NAVIGATION_ARROWS ? TRUE : FALSE, NULL);
+		action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionLastChunck");
+		g_object_set (G_OBJECT (action), "sensitive", 
+			      sample_last_row < row_count - 1 ? TRUE : FALSE,
+			      "visible", grid->priv->mode & GNOME_DB_ACTION_NAVIGATION_ARROWS ? TRUE : FALSE, NULL);
+	}
+	else {
 		/* disabled the chuncks navigation */
 		action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionFirstChunck");
 		g_object_set (G_OBJECT (action), "visible", FALSE, NULL);
@@ -1909,9 +2256,19 @@ modif_buttons_update (GnomeDbGrid *grid)
 		g_object_set (G_OBJECT (action), "visible", FALSE, NULL);
 		action = gtk_ui_manager_get_action (grid->priv->uimanager, "/ToolBar/ActionLastChunck");
 		g_object_set (G_OBJECT (action), "visible", FALSE, NULL);
-
-		gtk_widget_hide (grid->priv->widget_current_sample);
 	}
+	
+	if (sample_last_row == 0)
+		str = g_strdup_printf (_("0 - 0 / 0"));
+	else
+		str = g_strdup_printf ("%d - %d / %d", sample_first_row + 1, sample_last_row + 1, row_count);
+	gtk_label_set_text (GTK_LABEL (grid->priv->widget_current_sample), str);
+	g_free (str);
+	gtk_widget_set_sensitive (grid->priv->widget_current_sample, TRUE);
+	if (grid->priv->mode & GNOME_DB_ACTION_NAVIGATION_ARROWS)
+		gtk_widget_show (grid->priv->widget_current_sample);
+	else
+		gtk_widget_hide (grid->priv->widget_current_sample);
 
 	gtk_ui_manager_ensure_update (grid->priv->uimanager); 
 }
@@ -1933,10 +2290,19 @@ action_delete_cb (GtkAction *action, GnomeDbGrid *grid)
 {
 	GtkTreeIter iter;
 	GtkTreeSelection *select;
-
+	GtkTreeModel *model;
+	GList *sel_rows, *cur_row;
+	
 	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid->priv->treeview));
-	if (gtk_tree_selection_get_selected (select, NULL, &iter)) 
+	sel_rows = gtk_tree_selection_get_selected_rows (select, &model);
+	cur_row = sel_rows;
+	while (cur_row) {
+		gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) (cur_row->data));
 		gnome_db_data_proxy_delete (grid->priv->proxy, &iter);
+		cur_row = g_list_next (cur_row);
+	}
+	g_list_foreach (sel_rows, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (sel_rows);
 }
 
 static void
@@ -1944,10 +2310,19 @@ action_undelete_cb (GtkAction *action, GnomeDbGrid *grid)
 {
 	GtkTreeIter iter;
 	GtkTreeSelection *select;
-
+	GtkTreeModel *model;
+	GList *sel_rows, *cur_row;
+	
 	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid->priv->treeview));
-	if (gtk_tree_selection_get_selected (select, NULL, &iter)) 
+	sel_rows = gtk_tree_selection_get_selected_rows (select, &model);
+	cur_row = sel_rows;
+	while (cur_row) {
+		gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) (cur_row->data));
 		gnome_db_data_proxy_undelete (grid->priv->proxy, &iter);
+		cur_row = g_list_next (cur_row);
+	}
+	g_list_foreach (sel_rows, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (sel_rows);
 }
 
 static void
@@ -1979,19 +2354,33 @@ action_first_chunck_cb (GtkAction *action, GnomeDbGrid *grid)
 static void
 action_prev_chunck_cb (GtkAction *action, GnomeDbGrid *grid)
 {
-	gnome_db_grid_set_sample_start (grid, grid->priv->sample_first_row - grid->priv->sample_size);
+	gint sample_size, sample_start;
+
+	sample_size = gnome_db_data_proxy_get_sample_size (grid->priv->proxy);
+	if (sample_size > 0) {
+		sample_start = gnome_db_data_proxy_get_sample_start (grid->priv->proxy);
+		sample_start -= sample_size;
+		gnome_db_data_proxy_set_sample_start (grid->priv->proxy, sample_start);
+	}
 }
 
 static void
 action_next_chunck_cb (GtkAction *action, GnomeDbGrid *grid)
 {
-	gnome_db_grid_set_sample_start (grid, grid->priv->sample_first_row + grid->priv->sample_size);
+	gint sample_size, sample_start;
+
+	sample_size = gnome_db_data_proxy_get_sample_size (grid->priv->proxy);
+	if (sample_size > 0) {
+		sample_start = gnome_db_data_proxy_get_sample_start (grid->priv->proxy);
+		sample_start += sample_size;
+		gnome_db_data_proxy_set_sample_start (grid->priv->proxy, sample_start);
+	}
 }
 
 static void
 action_last_chunck_cb (GtkAction *action, GnomeDbGrid *grid)
 {
-	gnome_db_grid_set_sample_start (grid, G_MAXINT - grid->priv->sample_size);
+	gnome_db_data_proxy_set_sample_start (grid->priv->proxy, G_MAXINT);
 }
 
 
