@@ -1,8 +1,9 @@
 /* GNOME DB library
- * Copyright (C) 1999-2002 The GNOME Foundation.
+ * Copyright (C) 1999 - 2005 The GNOME Foundation.
  *
  * AUTHORS:
  * 	Rodrigo Moya <rodrigo@gnome-db.org>
+ *      Vivien Malerba <malerba@gnome-db.org>
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -20,9 +21,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifdef BUILD_WITH_GNOME
-#include <bonobo/bonobo-ui-main.h>
-#endif
 #ifdef HAVE_LIBGLADE
 #include <glade/glade-init.h>
 #endif
@@ -31,6 +29,7 @@
 #include <libgnomeui/gnome-ui-init.h>
 #endif
 #include <libgnomedb/libgnomedb.h>
+#include <gconf/gconf-client.h>
 
 extern void gnome_db_stock_init (void);
 GnomeDbDict *default_dict = NULL; /* available in all libgnomedb, anways NOT NULL */
@@ -75,20 +74,24 @@ gnome_db_init (const gchar *app_id, const gchar *version, gint nargs, gchar *arg
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
 	gda_init (app_id, version, nargs, args);
+
 #ifdef BUILD_WITH_GNOME
 	gnome_program_init (app_id, version, LIBGNOMEUI_MODULE, nargs, args,
 			    GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 
-	if (!bonobo_ui_is_initialized ())
-		bonobo_ui_init (app_id, version, &nargs, args);
 #else
-	gtk_init (nargs, args);
+	gtk_init (&nargs, &args);
 #endif
 
 #ifdef HAVE_LIBGLADE
 	glade_init ();
 #endif
 	gnome_db_stock_init ();
+
+	/* init GConf */
+	if (!gconf_is_initialized ())
+		gconf_init (0, NULL, NULL);
+	
 	default_dict = GNOME_DB_DICT (gnome_db_dict_new ());
 }
 
@@ -117,10 +120,6 @@ idle_cb (gpointer user_data)
  * @init_func: function to be called when everything has been initialized
  * @user_data: data to be passed to the init function
  *
- * Runs the GNOME-DB main loop, which is nothing more than the Bonobo UI main
- * loop, but with internally added stuff specific for applications using
- * libgda/libgnomedb.
- *
  * You can specify a function to be called after everything has been correctly
  * initialized (that is, for initializing your own stuff).
  */
@@ -137,11 +136,7 @@ gnome_db_main_run (GdaInitFunc init_func, gpointer user_data)
 		g_idle_add ((GSourceFunc) idle_cb, cb_data);
 	}
 
-#ifdef BUILD_WITH_GNOME
-	bonobo_ui_main ();
-#else
 	gtk_main();
-#endif	
 }
 
 /**
