@@ -1,6 +1,6 @@
 /* gnome-db-qf-func.c
  *
- * Copyright (C) 2003 - 2004 Vivien Malerba
+ * Copyright (C) 2003 - 2005 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -673,7 +673,7 @@ gnome_db_qf_func_dump (GnomeDbQfFunc *func, guint offset)
 
 
 /* 
- * GnomeDbFunc interface implementation
+ * GnomeDbEntity interface implementation
  */
 static GnomeDbEntity *
 gnome_db_qf_func_get_entity (GnomeDbField *iface)
@@ -789,7 +789,7 @@ gnome_db_qf_func_load_from_xml (GnomeDbXmlStorage *iface, xmlNodePtr node, GErro
 			g_set_error (error,
 				     GNOME_DB_QF_FUNC_ERROR,
 				     GNOME_DB_QF_FUNC_XML_LOAD_ERROR,
-				     _("Wrong type of func in <GNOME_DB_QF>"));
+				     _("Wrong type of function in <GNOME_DB_QF>"));
 			return FALSE;
 		}
 		g_free (prop);
@@ -1025,6 +1025,8 @@ gnome_db_qf_func_render_as_str (GnomeDbRenderer *iface, GnomeDbDataSet *context)
 
 			list = g_slist_next (list);
 		}
+		g_string_append (string, ")");
+
 		str = string->str;
 		g_string_free (string, FALSE);
 
@@ -1053,7 +1055,7 @@ gnome_db_qf_func_activate (GnomeDbReferer *iface)
 	active = gnome_db_ref_base_activate (func->priv->func_ref);
 	list = func->priv->args;
 	while (list) {
-		active = gnome_db_ref_base_activate (GNOME_DB_REF_BASE (list->data)) && active;
+		active = active && gnome_db_ref_base_activate (GNOME_DB_REF_BASE (list->data));
 		list = g_slist_next (list);
 	}
 
