@@ -44,7 +44,8 @@ enum
 {
 	GNOME_DB_QF_VALUE_XML_LOAD_ERROR,
 	GNOME_DB_QF_VALUE_RENDER_ERROR,
-	GNOME_DB_QF_VALUE_PARAM_ERROR
+	GNOME_DB_QF_VALUE_PARAM_ERROR,
+	GNOME_DB_QF_VALUE_DEFAULT_PARAM_ERROR
 };
 
 
