@@ -1,5 +1,5 @@
 /* GNOME DB library
- * Copyright (C) 1999-2002 The GNOME Foundation.
+ * Copyright (C) 1999 - 2005 The GNOME Foundation.
  *
  * AUTHORS:
  * 	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -24,7 +24,7 @@
 #include <gtk/gtktable.h>
 #include <libgnomedb/gnome-db-table-editor.h>
 #include <libgnomedb/gnome-db-util.h>
-#include "gnome-db-i18n.h"
+#include <glib/gi18n-lib.h>
 
 #define PARENT_TYPE GTK_TYPE_VBOX
 
