@@ -1,6 +1,6 @@
 /* gnome-db-table-field.c
  *
- * Copyright (C) 2003 - 2004 Vivien Malerba
+ * Copyright (C) 2003 - 2005 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -269,7 +269,7 @@ gnome_db_table_field_handler_func (GnomeDbServer *srv, GObject *obj)
 {
 	if (IS_GNOME_DB_TABLE_FIELD (obj)) 
 		return gnome_db_server_get_handler_by_gda (srv, 
-						     gnome_db_server_data_type_get_gda_type (GNOME_DB_TABLE_FIELD (obj)->priv->data_type));
+							   gnome_db_server_data_type_get_gda_type (GNOME_DB_TABLE_FIELD (obj)->priv->data_type));
 	else
 		return NULL;
 }
@@ -300,6 +300,7 @@ gnome_db_table_field_dispose (GObject *object)
 		if (gnome_db_table_field->priv->data_type) {
 			g_signal_handlers_disconnect_by_func (G_OBJECT (gnome_db_table_field->priv->data_type),
 							      G_CALLBACK (nullified_object_cb), gnome_db_table_field);
+			g_object_unref (gnome_db_table_field->priv->data_type);
 			gnome_db_table_field->priv->data_type = NULL;
 		}
 	}
@@ -502,9 +503,10 @@ gnome_db_table_field_set_data_type (GnomeDbTableField *field, GnomeDbServerDataT
 		g_signal_handlers_disconnect_by_func (G_OBJECT (field->priv->data_type),
 						      G_CALLBACK (nullified_object_cb), field);
 	field->priv->data_type = type;
+	g_object_ref (type);
 	gnome_db_base_connect_nullify (type, 
-				 G_CALLBACK (nullified_object_cb), field);
-
+				       G_CALLBACK (nullified_object_cb), field);
+	
 	/* signal the modification */
 	gnome_db_base_changed (GNOME_DB_BASE (field));
 }
@@ -840,9 +842,23 @@ gnome_db_table_field_load_from_xml (GnomeDbXmlStorage *iface, xmlNodePtr node, G
 
 	prop = xmlGetProp (node, "type");
 	if (prop) {
-		GnomeDbServerDataType *dt = gnome_db_server_get_data_type_by_xml_id (FIELD_SRV (field), prop);
-		if (dt) {
-			gnome_db_table_field_set_data_type (field, dt);
+		if ((*prop == 'D') && prop+1 && (*(prop+1) == 'T')) {
+			GnomeDbServerDataType *dt = gnome_db_server_get_data_type_by_xml_id (FIELD_SRV (field), prop);
+			if (dt) 
+				gnome_db_table_field_set_data_type (field, dt);
+			else {
+				/* create a new custom data type */
+				GnomeDbServer *srv;
+				
+				srv = gnome_db_dict_get_server (gnome_db_base_get_dict (GNOME_DB_BASE (field)));
+				dt = GNOME_DB_SERVER_DATA_TYPE (gnome_db_server_data_type_new (srv));
+				gnome_db_server_data_type_set_sqlname (dt, prop + 2);
+				gnome_db_server_data_type_set_gda_type (dt, GDA_VALUE_TYPE_BLOB);
+				gnome_db_base_set_description (GNOME_DB_BASE (dt), _("Custom data type"));
+				gnome_db_server_declare_custom_data_type (srv, dt);
+				gnome_db_table_field_set_data_type (field, dt);
+				g_object_unref (dt);
+			}
 			type = TRUE;
 		}
 		g_free (prop);
