@@ -650,13 +650,28 @@ gnome_db_table_update_dbms_data (GnomeDbTable *table, GError **error)
 		
 		/* Data type */
 		value = gda_data_model_get_value_at (rs, 1, now);
-		if (value && !gda_value_is_null (value) && (* gda_value_get_string(value))) {
+		if (value && !gda_value_is_null (value) && gda_value_get_string (value) && (* gda_value_get_string (value))) {
 			GnomeDbServerDataType *type;
 
 			str = gda_value_stringify (value);
 			type = gnome_db_server_get_data_type_by_name (srv, str);
 			if (type)
 				gnome_db_table_field_set_data_type (field, type);
+			else {
+				/* declare a custom data type */
+				gchar *descr;
+				type = gnome_db_server_data_type_new (srv);
+				gnome_db_server_data_type_set_sqlname (type, str);
+				gnome_db_server_data_type_set_gda_type (type, GDA_VALUE_TYPE_BLOB);
+				descr = g_strdup_printf (_("Custom data type, declared for the %s.%s field"),
+							 gnome_db_base_get_name (GNOME_DB_BASE (table)),
+							 gnome_db_base_get_name (GNOME_DB_BASE (field)));
+				gnome_db_base_set_description (GNOME_DB_BASE (type), descr);
+				g_free (descr);
+				gnome_db_server_declare_custom_data_type (srv, type);
+				gnome_db_table_field_set_data_type (field, type);
+				g_object_unref (type);
+			}
 			g_free (str);
 		}
 		if (!gnome_db_field_get_data_type (GNOME_DB_FIELD (field))) {
@@ -759,7 +774,7 @@ gnome_db_table_update_dbms_data (GnomeDbTable *table, GError **error)
 
 		/* FOREIGN KEY constraint */
 		value = gda_data_model_get_value_at (rs, 7, now);
-		if (value && !gda_value_is_null (value) && (* gda_value_get_string (value))) {
+		if (value && !gda_value_is_null (value) && gda_value_get_string (value) && (* gda_value_get_string (value))) {
 			gchar *ref_table, *str, *tok;
 			GnomeDbRefBase *ref;
 			GnomeDbConstraint *cstr = NULL;
