@@ -278,7 +278,7 @@ gnome_db_wrapper_field_get_data_type (GnomeDbField *iface)
 
 			info = gnome_db_server_get_server_info (srv);
 			if (info) {
-				for (i = 0; (i < info->value_nb_tests_types) && !type; i ++) {
+				for (i = 0; (i < info->value_nb_tests_types) && (!type); i ++) {
 					if (info->value_test_gda_types [i] == gda_column_get_gdatype (col_attrs))
 						type = info->value_test_data_types [i];
 				}
