@@ -279,8 +279,7 @@ gnome_db_entry_combo_new (GnomeDbDataSet *data_set, GnomeDbDataSetNode *node)
 	combo->priv->info = utility_data_set_node_info_create (data_set, node);
 
 	/* create the combo box itself */
-	entry = gnome_db_combo_new_with_model (gnome_db_base_get_dict (GNOME_DB_BASE (data_set)), 
-					       GDA_DATA_MODEL (node->data_for_param), 
+	entry = gnome_db_combo_new_with_model (GDA_DATA_MODEL (node->data_for_param), 
 					       combo->priv->info->shown_n_cols, combo->priv->info->shown_cols_index);
 	g_signal_connect (G_OBJECT (entry), "changed",
 			  G_CALLBACK (combo_contents_changed_cb), combo);
